package org.jtheque.primary.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;
import org.jtheque.primary.dao.able.IDaoBorrowers;
import org.jtheque.primary.dao.able.IDaoCountries;
import org.jtheque.primary.od.BorrowerImpl;
import org.jtheque.primary.od.abstraction.Borrower;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * Cette classe permet de grer la persistances des emprunteurs.
 *
 * @author Baptiste Wicht
 */
public final class DaoBorrowers extends GenericDao<BorrowerImpl> implements IDaoBorrowers {
    private final ParameterizedRowMapper<BorrowerImpl> rowMapper = new BorrowerRowMapper();
    private final QueryMapper queryMapper = new BorrowerQueryMapper();

    @Resource
    private IDaoPersistenceContext persistenceContext;

    @Resource
    private IDaoCountries daoCountries;

    /**
     * Construct a new DaoBorrowers.
     */
    public DaoBorrowers() {
        super(TABLE);
    }

    @Override
    public List<BorrowerImpl> getBorrowers() {
        return getAll();
    }

    @Override
    public BorrowerImpl getBorrower(int id) {
        return get(id);
    }

    @Override
    protected void loadCache() {
        List<BorrowerImpl> borrowers = persistenceContext.getSortedList(TABLE, rowMapper);

        for (BorrowerImpl borrower : borrowers) {
            getCache().put(borrower.getId(), borrower);
        }

        setCacheEntirelyLoaded(true);
    }

    @Override
    protected void load(int i) {
        BorrowerImpl borrower = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, borrower);
    }

    @Override
    protected ParameterizedRowMapper<BorrowerImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    /**
     * A row mapper to map resultset to borrower.
     *
     * @author Baptiste Wicht
     */
    private final class BorrowerRowMapper implements ParameterizedRowMapper<BorrowerImpl> {
        @Override
        public BorrowerImpl mapRow(ResultSet rs, int i) throws SQLException {
            BorrowerImpl borrower = new BorrowerImpl();

            borrower.setId(rs.getInt("ID"));
            borrower.setName(rs.getString("NAME"));
            borrower.setFirstName(rs.getString("FIRSTNAME"));
            borrower.setTheCountry(daoCountries.getCountry(rs.getInt("THE_COUNTRY_FK")));

            return borrower;
        }
    }

    /**
     * A query mapper to map borrower to query.
     *
     * @author Baptiste Wicht
     */
    private static final class BorrowerQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            Borrower borrower = (Borrower) entity;

            String query = "INSERT INTO " + TABLE + " (NAME, FIRSTNAME, THE_COUNTRY_FK, EMAIL) VALUES(?,?,?,?)";

            Object[] parameters = {
                    borrower.getName(),
                    borrower.getFirstName(),
                    borrower.getTheCountry() == null ? 0 : borrower.getTheCountry().getId(),
                    borrower.getEmail()};

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            Borrower borrower = (Borrower) entity;

            String query = "UPDATE " + TABLE + " SET NAME = ?, FIRSTNAME = ?, THE_COUNTRY_FK = ?, EMAIL = ? WHERE ID = ?";

            Object[] parameters = {
                    borrower.getName(),
                    borrower.getFirstName(),
                    borrower.getTheCountry() == null ? 0 : borrower.getTheCountry().getId(),
                    borrower.getEmail(),
                    borrower.getId()};

            return new Query(query, parameters);
        }
    }
}