package org.jtheque.primary.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;
import org.jtheque.primary.dao.able.IDaoCollections;
import org.jtheque.primary.od.CollectionImpl;
import org.jtheque.primary.od.KindImpl;
import org.jtheque.primary.od.abstraction.Collection;
import org.jtheque.primary.PrimaryUtils;
import org.jtheque.utils.StringUtils;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.ArrayList;

/**
 * Cette classe permet de grer la persistance des films.
 *
 * @author Baptiste Wicht
 */
public final class DaoCollections extends GenericDao<CollectionImpl> implements IDaoCollections {
    private final ParameterizedRowMapper<CollectionImpl> rowMapper = new CollectionRowMapper();
    private final QueryMapper queryMapper = new CollectionQueryMapper();

    @Resource
    private IDaoPersistenceContext persistenceContext;

    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    /**
     * The current collection.
     */
    private CollectionImpl currentCollection;

    /**
     * Construct a new DaoCollections.
     */
    public DaoCollections() {
        super(TABLE);
    }

    @Override
    public void create(CollectionImpl entity) {
        entity.setPrimaryImpl(PrimaryUtils.getPrimaryImpl());
        
        super.create(entity);
    }

    @Override
    public List<CollectionImpl> getCollections() {
        return getCollections(PrimaryUtils.getPrimaryImpl());
    }

    private List<CollectionImpl> getCollections(String impl) {
        if (StringUtils.isEmpty(impl)) {
            return getAll();
        }

        load();

        List<CollectionImpl> collections = new ArrayList<CollectionImpl>(getCache().size() / 2);

        for (CollectionImpl collection : getCache().values()) {
            if (impl.equals(collection.getPrimaryImpl())) {
                collections.add(collection);
            }
        }

        return collections;
    }

    @Override
    public CollectionImpl getCollection(int id) {
        return get(id);
    }

    @Override
    public CollectionImpl getCollection(String name){
        List<CollectionImpl> collections = jdbcTemplate.query("SELECT * FROM " + TABLE + " WHERE TITLE = ? AND IMPL = ?", 
                rowMapper, name, PrimaryUtils.getPrimaryImpl());

        if(collections.isEmpty()){
            return null;
        }

        CollectionImpl collection = collections.get(0);

        if (isNotInCache(collection.getId())) {
            getCache().put(collection.getId(), collection);
        }

        return getCache().get(collection.getId());
    }

    @Override
    protected ParameterizedRowMapper<CollectionImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    @Override
    protected void loadCache() {
        List<CollectionImpl> collections = persistenceContext.getSortedList(TABLE, rowMapper);

        for (CollectionImpl collection : collections) {
            getCache().put(collection.getId(), collection);
        }

        setCacheEntirelyLoaded(true);
    }

    @Override
    protected void load(int i) {
        CollectionImpl collection = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, collection);
    }

    @Override
    public CollectionImpl getCurrentCollection() {
        return currentCollection;
    }

    @Override
    public void setCurrentCollection(CollectionImpl collection) {
        currentCollection = collection;
    }

    /**
     * A row mapper to map resultset to collection.
     *
     * @author Baptiste Wicht
     */
    private static final class CollectionRowMapper implements ParameterizedRowMapper<CollectionImpl> {
        @Override
        public CollectionImpl mapRow(ResultSet rs, int i) throws SQLException {
            CollectionImpl collection = new CollectionImpl();

            collection.setId(rs.getInt("ID"));
            collection.setTitle(rs.getString("TITLE"));
            collection.setPassword(rs.getString("PASSWORD"));
            collection.setProtection(rs.getBoolean("PROTECTED"));
            collection.setPrimaryImpl(rs.getString("IMPL"));

            return collection;
        }
    }

    /**
     * A query mapper to map collection to sql query.
     *
     * @author Baptiste Wicht
     */
    private static final class CollectionQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            Collection collection = (Collection) entity;

            String query = "INSERT INTO " + TABLE + " (TITLE, PASSWORD, PROTECTED, IMPL) VALUES(?,?,?,?)";

            Object[] parameters = {
                    collection.getTitle(),
                    collection.getPassword(),
                    collection.isProtection(),
                    collection.getPrimaryImpl()
            };

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            Collection collection = (Collection) entity;

            String query = "UPDATE " + TABLE + " SET TITLE = ?, PASSWORD = ?, PROTECTED = ?, IMPL = ? WHERE ID = ?";

            Object[] parameters = {
                    collection.getTitle(),
                    collection.getPassword(),
                    collection.isProtection(),
                    collection.getPrimaryImpl(),
                    collection.getId()
            };

            return new Query(query, parameters);
        }
    }
}