package org.jtheque.primary.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.primary.dao.able.IDaoLendings;
import org.jtheque.primary.od.LendingImpl;
import org.jtheque.primary.od.TypeImpl;
import org.jtheque.primary.od.SagaImpl;
import org.jtheque.primary.od.abstraction.Lending;
import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;
import org.jtheque.utils.bean.IntDate;
import org.jtheque.utils.StringUtils;
import org.jtheque.primary.dao.able.IDaoBorrowers;
import org.jtheque.primary.PrimaryUtils;

import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.ArrayList;

/**
 * Cette classe permet de grer la persistance des emprunts.
 *
 * @author Baptiste Wicht
 */
public final class DaoLendings extends GenericDao<LendingImpl> implements IDaoLendings {
    private final ParameterizedRowMapper<LendingImpl> rowMapper = new LendingRowMapper();
    private final QueryMapper queryMapper = new LendingQueryMapper();

    @Resource
    private IDaoPersistenceContext persistenceContext;

    @Resource
    private SimpleJdbcTemplate jdbcTemplate;

    @Resource
    private IDaoBorrowers daoBorrowers;

    /**
     * Construct a new DaoLendings.
     */
    public DaoLendings() {
        super(TABLE);
    }

    @Override
    public void create(LendingImpl entity) {
        entity.setPrimaryImpl(PrimaryUtils.getPrimaryImpl());
        
        super.create(entity);
    }

    @Override
    public LendingImpl getLending(int id) {
        return get(id);
    }

    @Override
    protected ParameterizedRowMapper<LendingImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    @Override
    protected void loadCache() {
        List<LendingImpl> lendings = persistenceContext.getSortedList(TABLE, rowMapper);

        for (LendingImpl lending : lendings) {
            getCache().put(lending.getId(), lending);
        }

        setCacheEntirelyLoaded(true);
    }

    @Override
    public List<LendingImpl> getLendings() {
        return getLendings(PrimaryUtils.getPrimaryImpl());
    }

    private List<LendingImpl> getLendings(String impl) {
        if (StringUtils.isEmpty(impl)) {
            return getAll();
        }

        load();

        List<LendingImpl> lendings = new ArrayList<LendingImpl>(getCache().size() / 2);

        for (LendingImpl lending : getCache().values()) {
            if (impl.equals(lending.getPrimaryImpl())) {
                lendings.add(lending);
            }
        }

        return lendings;
    }

    @Override
    protected void load(int i) {
        LendingImpl lending = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, lending);
    }

    /**
     * A mapper to map resultset to lending.
     *
     * @author Baptiste Wicht
     */
    private final class LendingRowMapper implements ParameterizedRowMapper<LendingImpl> {
        @Override
        public LendingImpl mapRow(ResultSet rs, int i) throws SQLException {
            LendingImpl lending = new LendingImpl();

            lending.setId(rs.getInt("ID"));
            lending.setTheOther(rs.getInt("THE_OTHER_FK"));
            lending.setTheBorrower(daoBorrowers.getBorrower(rs.getInt("THE_BORROWER_FK")));
            lending.setDate(new IntDate(rs.getInt("DATE")));
            lending.setPrimaryImpl(rs.getString("IMPL"));

            return lending;
        }
    }

    /**
     * A mapper to map lending to query.
     *
     * @author Baptiste Wicht
     */
    private static final class LendingQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            Lending lending = (Lending) entity;

            String query = "INSERT INTO " + TABLE + " (THE_OTHER_FK, THE_BORROWER_FK, DATE, IMPL) VALUES(?,?,?, ?)";

            Object[] parameters = {
                    lending.getTheOther(),
                    lending.getTheBorrower().getId(),
                    lending.getDate().intValue(), 
                    lending.getPrimaryImpl()
            };

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            Lending lending = (Lending) entity;

            String query = "UPDATE " + TABLE + " SET THE_OTHER_FK = ?, THE_BORROWER_FK = ?, DATE = ?, IMPL = ? WHERE ID = ?";

            Object[] parameters = {
                    lending.getTheOther(),
                    lending.getTheBorrower().getId(),
                    lending.getDate().intValue(),
                    lending.getPrimaryImpl(),
                    lending.getId()};

            return new Query(query, parameters);
        }
    }
}